package java.security.cert;

import java.util.Iterator;
import java.security.KeyStoreException;
import java.util.Enumeration;
import java.util.HashSet;
import java.security.KeyStore;
import java.security.InvalidAlgorithmParameterException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Date;
import java.util.Set;

public class PKIXParameters implements CertPathParameters
{
    private Set<TrustAnchor> unmodTrustAnchors;
    private Date date;
    private List<PKIXCertPathChecker> certPathCheckers;
    private String sigProvider;
    private boolean revocationEnabled;
    private Set<String> unmodInitialPolicies;
    private boolean explicitPolicyRequired;
    private boolean policyMappingInhibited;
    private boolean anyPolicyInhibited;
    private boolean policyQualifiersRejected;
    private List<CertStore> certStores;
    private CertSelector certSelector;
    
    public PKIXParameters(Set<TrustAnchor> trustAnchors) throws InvalidAlgorithmParameterException {
        this.revocationEnabled = true;
        this.explicitPolicyRequired = false;
        this.policyMappingInhibited = false;
        this.anyPolicyInhibited = false;
        this.policyQualifiersRejected = true;
        this.setTrustAnchors(trustAnchors);
        this.unmodInitialPolicies = Collections.emptySet();
        this.certPathCheckers = new ArrayList();
        this.certStores = new ArrayList();
    }
    
    public PKIXParameters(KeyStore keyStore) throws KeyStoreException, InvalidAlgorithmParameterException {
        this.revocationEnabled = true;
        this.explicitPolicyRequired = false;
        this.policyMappingInhibited = false;
        this.anyPolicyInhibited = false;
        this.policyQualifiersRejected = true;
        if (keyStore == null) {
            throw new NullPointerException("the keystore parameter must be non-null");
        }
        HashSet<TrustAnchor> trustAnchors = new HashSet();
        Enumeration<String> aliases = keyStore.aliases();
        while (aliases.hasMoreElements()) {
            String s = (String)aliases.nextElement();
            if (keyStore.isCertificateEntry(s)) {
                Certificate certificate = keyStore.getCertificate(s);
                if ((certificate instanceof X509Certificate)) {
                    trustAnchors.add(new TrustAnchor((X509Certificate)certificate, null));
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
        this.setTrustAnchors(trustAnchors);
        this.unmodInitialPolicies = Collections.emptySet();
        this.certPathCheckers = new ArrayList();
        this.certStores = new ArrayList();
        return;
    }
    
    public Set<TrustAnchor> getTrustAnchors() {
        return this.unmodTrustAnchors;
    }
    
    public void setTrustAnchors(Set<TrustAnchor> c) throws InvalidAlgorithmParameterException {
        if (c == null) {
            throw new NullPointerException("the trustAnchors parameters must be non-null");
        }
        if (c.isEmpty()) {
            throw new InvalidAlgorithmParameterException("the trustAnchors parameter must be non-empty");
        }
        Iterator<Object> iterator = c.iterator();
        while (iterator.hasNext()) {
            if (iterator.next() instanceof TrustAnchor) {
                continue;
            }
            throw new ClassCastException("all elements of set must be of type java.security.cert.TrustAnchor");
        }
        this.unmodTrustAnchors = Collections.unmodifiableSet(new HashSet(c));
        return;
    }
    
    public Set<String> getInitialPolicies() {
        return this.unmodInitialPolicies;
    }
    
    public void setInitialPolicies(Set<String> c) {
        if (c != null) {
            Iterator<Object> iterator = c.iterator();
            while (iterator.hasNext()) {
                if (iterator.next() instanceof String) {
                    continue;
                }
                throw new ClassCastException("all elements of set must be of type java.lang.String");
            }
            this.unmodInitialPolicies = Collections.unmodifiableSet(new HashSet(c));
        }
        else {
            this.unmodInitialPolicies = Collections.emptySet();
        }
    }
    
    public void setCertStores(List<CertStore> c) {
        if (c == null) {
            this.certStores = new ArrayList();
        }
        else {
            Iterator<Object> iterator = c.iterator();
            while (iterator.hasNext()) {
                if (iterator.next() instanceof CertStore) {
                    continue;
                }
                throw new ClassCastException("all elements of list must be of type java.security.cert.CertStore");
            }
            this.certStores = new ArrayList(c);
        }
    }
    
    public void addCertStore(CertStore certStore) {
        if (certStore != null) {
            this.certStores.add(certStore);
        }
    }
    
    public List<CertStore> getCertStores() {
        return Collections.unmodifiableList(new ArrayList(this.certStores));
    }
    
    public void setRevocationEnabled(boolean revocationEnabled) {
        this.revocationEnabled = revocationEnabled;
    }
    
    public boolean isRevocationEnabled() {
        return this.revocationEnabled;
    }
    
    public void setExplicitPolicyRequired(boolean explicitPolicyRequired) {
        this.explicitPolicyRequired = explicitPolicyRequired;
    }
    
    public boolean isExplicitPolicyRequired() {
        return this.explicitPolicyRequired;
    }
    
    public void setPolicyMappingInhibited(boolean policyMappingInhibited) {
        this.policyMappingInhibited = policyMappingInhibited;
    }
    
    public boolean isPolicyMappingInhibited() {
        return this.policyMappingInhibited;
    }
    
    public void setAnyPolicyInhibited(boolean anyPolicyInhibited) {
        this.anyPolicyInhibited = anyPolicyInhibited;
    }
    
    public boolean isAnyPolicyInhibited() {
        return this.anyPolicyInhibited;
    }
    
    public void setPolicyQualifiersRejected(boolean policyQualifiersRejected) {
        this.policyQualifiersRejected = policyQualifiersRejected;
    }
    
    public boolean getPolicyQualifiersRejected() {
        return this.policyQualifiersRejected;
    }
    
    public Date getDate() {
        if (this.date == null) {
            return null;
        }
        return (Date)this.date.clone();
    }
    
    public void setDate(Date date) {
        if (date != null) {
            this.date = (Date)date.clone();
        }
    }
    
    public void setCertPathCheckers(List<PKIXCertPathChecker> list) {
        if (list != null) {
            ArrayList<E> certPathCheckers = new ArrayList();
            Iterator<Object> iterator = list.iterator();
            while (iterator.hasNext()) {
                certPathCheckers.add((E)((PKIXCertPathChecker)iterator.next()).clone());
            }
            this.certPathCheckers = certPathCheckers;
        }
        else {
            this.certPathCheckers = new ArrayList();
        }
    }
    
    public List<PKIXCertPathChecker> getCertPathCheckers() {
        ArrayList<E> list = new ArrayList();
        Iterator<PKIXCertPathChecker> iterator = this.certPathCheckers.iterator();
        while (iterator.hasNext()) {
            list.add((E)((PKIXCertPathChecker)iterator.next()).clone());
        }
        return Collections.unmodifiableList(list);
    }
    
    public void addCertPathChecker(PKIXCertPathChecker pkixCertPathChecker) {
        if (pkixCertPathChecker != null) {
            this.certPathCheckers.add((PKIXCertPathChecker)pkixCertPathChecker.clone());
        }
    }
    
    public String getSigProvider() {
        return this.sigProvider;
    }
    
    public void setSigProvider(String sigProvider) {
        this.sigProvider = sigProvider;
    }
    
    public CertSelector getTargetCertConstraints() {
        if (this.certSelector != null) {
            return (CertSelector)this.certSelector.clone();
        }
        return null;
    }
    
    public void setTargetCertConstraints(CertSelector certSelector) {
        if (certSelector != null) {
            this.certSelector = (CertSelector)certSelector.clone();
        }
        else {
            this.certSelector = null;
        }
    }
    
    @Override
    public Object clone() {
        try {
            PKIXParameters pkixParameters = (PKIXParameters)super.clone();
            if (this.certStores != null) {
                pkixParameters.certStores = new ArrayList(this.certStores);
            }
            if (this.certPathCheckers != null) {
                pkixParameters.certPathCheckers = new ArrayList(this.certPathCheckers.size());
                Iterator<PKIXCertPathChecker> iterator = this.certPathCheckers.iterator();
                while (iterator.hasNext()) {
                    pkixParameters.certPathCheckers.add((PKIXCertPathChecker)((PKIXCertPathChecker)iterator.next()).clone());
                }
                return pkixParameters;
            }
            return pkixParameters;
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause.toString(), cause);
        }
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("[\n");
        if (this.unmodTrustAnchors != null) {
            sb.append("  Trust Anchors: " + this.unmodTrustAnchors.toString() + "\n");
        }
        if (this.unmodInitialPolicies != null) {
            if (this.unmodInitialPolicies.isEmpty()) {
                sb.append("  Initial Policy OIDs: any\n");
            }
            else {
                sb.append("  Initial Policy OIDs: [" + this.unmodInitialPolicies.toString() + "]\n");
            }
        }
        sb.append("  Validity Date: " + String.valueOf(this.date) + "\n");
        sb.append("  Signature Provider: " + this.sigProvider + "\n");
        sb.append("  Default Revocation Enabled: " + this.revocationEnabled + "\n");
        sb.append("  Explicit Policy Required: " + this.explicitPolicyRequired + "\n");
        sb.append("  Policy Mapping Inhibited: " + this.policyMappingInhibited + "\n");
        sb.append("  Any Policy Inhibited: " + this.anyPolicyInhibited + "\n");
        sb.append("  Policy Qualifiers Rejected: " + this.policyQualifiersRejected + "\n");
        sb.append("  Target Cert Constraints: " + String.valueOf(this.certSelector) + "\n");
        if (this.certPathCheckers != null) {
            sb.append("  Certification Path Checkers: [" + this.certPathCheckers.toString() + "]\n");
        }
        if (this.certStores != null) {
            sb.append("  CertStores: [" + this.certStores.toString() + "]\n");
        }
        sb.append("]");
        return sb.toString();
    }
}
